#include "CParams.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <mpi.h>
#include "generic.h"
#include <iostream>
using namespace std;

CParams::CParams()
{	
	mtype 	= 0;
	etype	= 0;
	iniStr(MatrixFilePath, MAX_PATH_LENGTH);
	iniStr(DBFile, MAX_PATH_LENGTH);
	iniStr(seqQueryFile, MAX_PATH_LENGTH);
	iniStr(seqClsFile, MAX_PATH_LENGTH);
	shuffles_ML 	= DEFAULT_SHUFFLE;
}
CParams::~CParams()
{

}
void CParams::getSysTime(double *dtime)
{
  struct timeval tv;

  gettimeofday (&tv, NULL);

  *dtime = (double) tv.tv_sec;
  *dtime = *dtime + (double) (tv.tv_usec) / 1000000.0;
}
void CParams::printUsage()
{
    fprintf(stderr,"\n---------------------------------------------------------------\n");
	fprintf(stderr,"\tVersionDate: 06/20/11\n");
	fprintf(stderr,"\tSSM: Standard substitution matrix\n");
	fprintf(stderr,"\tPSSM: Position-specific substitution matrix\n");
	fprintf(stderr,"\n---------------------------------------------------------------\n");	
	fprintf(stderr,"\nUsage:\n");
    fprintf(stderr,"./EXENAME [options]\n\n");

	fprintf(stderr,"\t-query <string>\t: specify sequence queryname file\n");
	fprintf(stderr,"\t-db <string>\t: specify database \n");
	fprintf(stderr,"\t-cls <string>\t: specify classification file \n");
	fprintf(stderr,"\t-mtype <integer>\t: specify the type of matrix\n");
	fprintf(stderr,"\t\t 0: SSM, 1: PSSM\n");
	fprintf(stderr,"\t-mat <string>\t: specify the path for substitution matrix \n");
	fprintf(stderr,"\t\tFor SSM, need to specify the file name\n");
	fprintf(stderr,"\t\tFor PSSM, JUST need specify the path that include PSSM files\n");
	fprintf(stderr,"\t-etype <integer>\t: specify the type of estimation \n");
	fprintf(stderr,"\t\t 0 : for only normal; 1 : for only non-conservative.\n");
	fprintf(stderr,"\t\t 2 : conservative pairwise statistical significance\n");
    fprintf(stderr,"\t\t 3 : for average statistical significance\n");		
	fprintf(stderr,"\t-num <integer>\t: specify number of shuffles (default: 1000)\n");

}
bool CParams::parseParams(int argc, char* argv[])
{
	bool DBAvail 			= false;
	bool MatrixPathAvail 	= false;
	bool QueryAvail 		= false;
	bool ClsAvail 			= false;

	int pRank = MPI::COMM_WORLD.Get_rank();
	int index;
	int number;
	//display usages
	if (argc < 2 || !strcmp(argv[1], "-help") || !strcmp(argv[1], "-?")){		
		if (pRank == 0){
			printUsage();
		}
		return false;
	}
	index = 1;
	// parse other arguments
	char* arg;
	for ( ; index < argc; index++) {
		arg = argv[index];	
		if (index >= argc){
			if (pRank == 0){
				fprintf(stderr, "The number of the specified argments does not match!");
			}
			return false;
		}		
		
		if (strcmp(arg, "-query") == 0) {
			iniStr(seqQueryFile, MAX_PATH_LENGTH); 
			sscanf(argv[++index], "%s", seqQueryFile);
			QueryAvail = true;
		} else if (strcmp(arg, "-db") == 0) {
			iniStr(DBFile, MAX_PATH_LENGTH);
			sscanf(argv[++index], "%s", DBFile);
			DBAvail = true;
		} else if (strcmp(arg, "-cls") == 0) {
			iniStr(seqClsFile, MAX_PATH_LENGTH);
			sscanf(argv[++index], "%s", seqClsFile);
			ClsAvail = true;
		}else if (strcmp(arg, "-mtype") == 0) {
			sscanf(argv[++index], "%d", &number);
			if(number == 0 || number == 1){
				mtype = number;
			} else {
				if (pRank == 0){
					fprintf(stderr, "The type of matrix does not match!");
					printUsage();
				}
				return false;
			}
						
		} else if (strcmp(arg, "-mat") == 0) {
			sscanf(argv[++index], "%s", MatrixFilePath);
			MatrixPathAvail = true;
		} else if (strcmp(arg, "-etype") == 0) {
			sscanf(argv[++index], "%d", &number);
			if ( number != 1 && number != 0 && number != 2 && number != 3 ) {
				if (pRank == 0){
					printf("\nError : the type of estimation should be one of 0, 1 or 2.");	
					printUsage();
				}
				return false;
			}
			etype = number;
		} else if (strcmp(arg, "-num") == 0) {
			sscanf(argv[++index],"%d", &number);
			if ( number < 1 || number > 10000) {
				printf("\nThe number of shuffles ('-num') should be a 1 ~ 10000.\n");
				printf("\tusing default shuffles: %d ",DEFAULT_SHUFFLE);				
				shuffles_ML = DEFAULT_SHUFFLE;
			}
			shuffles_ML = number;	
        } else if (strcmp(arg, "-help") == 0 || strcmp(arg, "-?") == 0) {
            if (pRank == 0){
				printUsage();
            }
			return false;
        }else {
            if (pRank == 0){
			fprintf(stderr,"\n************************************\n");
            fprintf(stderr,"Unknown option: %s\n", arg);
            fprintf(stderr,"\n************************************\n");
            printUsage();
            }
			return false;
        }
	}
	if(DBAvail == false){
		if (pRank == 0){
			fprintf(stderr, "Please specifiy the database (subject sequences) file\n");
			printUsage();
			return false;
		}
	}
	if(QueryAvail == false){
		if (pRank == 0){
			fprintf(stderr, "Please specifiy the query filename File\n");
			printUsage();
			return false;
		}
	}
	if(MatrixPathAvail == false){
		if (pRank == 0){
			fprintf(stderr, "Please specifiy the file path of matrix \n");
			printUsage();
			return false;
		}
	}
	if(ClsAvail == false){
		if (pRank == 0){
			fprintf(stderr, "Please specifiy the classification file \n");
			printUsage();
			return false;
		}
	}
	
	return true;
}

