#ifndef _MPIPAIRWISE_
#define _MPIPAIRWISE_
#include <CParams.h>

static const int MaxNameLen	= 100;
static const int MaxSeqLen = 3000;

typedef struct tagSeqInfo{
	int length;
	char name[MaxNameLen];
	char context[MaxSeqLen];
	int cls[4]; //information about classification
}SeqInfo;

typedef struct pwCompRec {
	int seq1;
	char name1[MaxNameLen];
	int seq2;
	char name2[MaxNameLen];
	int topolog;
	int homolog;
	double lambda;
	double K;
	double score;
	double Pvalue;
} CompRec;

typedef struct multPavlueInfo_tag {
	double lambda1;
	double K1;
	double score1;
	double Evalue1;

	double lambda2;
	double K2;
	double score2;
	double Pvalue2;
	
} MPavlueInfo;

class StatSig {

public:
		StatSig(CParams* params);
		~StatSig();
		
		void run();
		void StatSigForSeqPair_ML(char* query, char* queryName, int queryLen,
								   char* subject, char* subName, int subLen); 

		void ShuffleSeq(char seq[], int len);
		void PrintResult();
		void ReadSeqs();
		int  loadClsOfSeqs();
		void GetPSSE();

		void ReadPSSMFromFile(char* PSSMFilename, int queryLen);
		void WritePSSToFile (CompRec* PSSInfo, int num, char* EvalueScvFile);
		void getMatrixFromFile(char* BlosumFilename,	int matrix[128][128]);
		void getLocalProfileFromSSM(char* query, int queryLen);
		
		static int	compar_ascent_Pvalue(const void* va, const void* vb);
		static int compar_descent_Pvalue(const void* va, const void* vb);
		
		void writePSSToFile (CompRec* PSSInfo, int num, char* EvalueScvFile);
		
		
		
private:
	int FindStringIndex(char* SeqName, SeqInfo* dbSeqs, int dbSeqNum);	
	char MatFilename[MAX_PATH_LENGTH]; //matrix path
	
	
	int 	Etype; /* the type of estimation */
	int		Mtype; /* the type of matrix */

	int shuffles_ML;			//number of scores to be used to generate empirical score distribution
	char* MatrixFilePath;
	char* DBFile;
	char* seqQueryFile;
	char* seqClsFile; 
		
	// for reading standard substution matrix.	
	static int BlmMatrix[128][128];
	
	// Pointer of resource Files
	FILE* fp_DB;
	FILE* fp_querySeq;
	
	// query and subject sequences information
	SeqInfo* DBSeqs;
	SeqInfo* QuerySeqs;
	int** dbSeqClf;
	
	double 	lambda_ML_censor[2], 	//returned statistical parameter lambda
			K_ML_censor[2], 		//returned statistical parameter K
			pwAlgnScore[2], 		//returned pairwise alignment score
			pwStatSig[2]; 			//returned pairwise statistical significance


	int gapOpen;
	int gapExten;
	
	int TotalSeqs; // total number of subject sequences
	int TotalQuery; // total number of query sequences
	
	double 	totalTime,
			IOTime, 
			alnTime, 
			gatherTime, 
			fitTime;
	

	int  lumpsize ;	  /*for alocating histogram*/
	int  lowscore;	  /* a low bound on the score of a selected pair */
	int  highscore;   /* upbound on scores of pairs of shuffled sequences */
	int  no_entry;     /*  1000  no. of entries in the prob distribution */
	
	int nProcessors,  //no. of processors
		pRank;		 //rank of processor	
		
	CParams* params;
	
	int* PSSMatrix;
	int* LocalMatrix; // for local profile matrix generate for SSM;

	int* amino_acids_trans;
	char* amino_acids;


};
			
#endif
