#ifndef _GENEIC_H_
#define _GENEIC_H_
#include <stddef.h>


static  int DEFAULT_SHUFFUL_NUM = 1000;

static  int MAX_AMINO_ACIDS 		= 23;
static const int DUMMY_AMINO_ACID 	= MAX_AMINO_ACIDS;
static const int MATRIX_SIZE 		= (MAX_AMINO_ACIDS + 1);

const static int   NO_CENSORING 					= 0;
const static int   CENSOR_LEFT_OF_PEAK 				= 1;
const static int   CENSOR_LEFT_OF_GIVEN_CUTOFF 		= 2;
#define MIN(a,b) ((a)<(b) ? (a) : (b))
#define MAX(a,b) ((a)>(b) ? (a) : (b))

//to check here if work? this macro just help to dedug--reduce and lock the error space.	

#define IFWORK { printf("Work here: %s, line  %d \n", __FILE__ ,__LINE__); }
#define CHECK(x) { cout <<__FILE__<< "  "<<__LINE__<<": the variable checked '" <<#x <<"' = "<<x<<endl;}   	

#ifdef DEBUG

#define ERROR_OUT(reason) \
	do{ { \
				fprintf(stdout, "\n%s, line  %d, %s\n",__FILE__ ,__LINE__, reason); \
				fflush(stdout); \
				
				exit(-1); \
	}}while(0)
#else

#define ERROR_OUT(reason) \
	do{ { \
				fprintf(stdout, "\n%s\n", reason); \
				fflush(stdout); \
				exit(-1); \
	}}while(0)

#endif

/* Function:  ckalloc(bytes)
 *
 * Descrip:    Tries to alloc bytes of memory. Posts
 *             to warn if it fails
 *
 *
 * Arg:        bytes [UNKN ] Undocumented argument [size_t]
 *
 * Return [UNKN ]  Undocumented return value [void *]
 *
 */
void* ckalloc(size_t bytes);



/* Function:  ckcalloc(len,bytes)
 *
 * Descrip:    calloc equivalent
 *
 *
 * Arg:          len [UNKN ] Undocumented argument [int]
 * Arg:        bytes [UNKN ] Undocumented argument [size_t]
 *
 * Return [UNKN ]  Undocumented return value [void *]
 *
 */
void* ckcalloc(int len,size_t bytes);



/* Function:  ckrealloc(*ptr,bytes)
 *
 * Descrip:    realloc equivalent
 *
 *
 * Arg:         *ptr [UNKN ] Undocumented argument [void]
 * Arg:        bytes [UNKN ] Undocumented argument [size_t]
 *
 * Return [UNKN ]  Undocumented return value [void *]
 *
 */
void* ckrealloc(void *ptr, size_t bytes);



/* Function:  ckfree(*ptr)
 *
 * Descrip:    free equivalent
 *
 *
 * Arg:        *ptr [UNKN ] Undocumented argument [void]
 *
 * Return [UNKN ]  Undocumented return value [void *]
 *
 */
void* ckfree(void *ptr);

void iniStr(char* s, int len);

/* fatal - print message and die */
void fatal(const char* msg);


/* fatalf - format message, print it, and die */
void fatalf(const char* msg, char* val);
	
/* ckopen - open file; check for success */
//FILE* ckopen(char* name, char* mode);


/* ckalloc1 - allocate space; check for success */
void *ckalloc1(size_t  amount);

double 		getSysTime();



// set size of acids amino acid residue is 23
static  char amino_acids_23[23] = {
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
    'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R',
    'S', 'T', 'V', 'W', 'X', 'Y', 'Z'
};

static int amino_acids_trans_23[256] = {
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1,  0,  1,  2,  3,  4,  5,  6,  7,  8, -1,  9, 10, 11, 12, -1,
    13, 14, 15, 16, 17, -1, 18, 19, 20, 21, 22, -1, -1, -1, -1, -1,
    -1,  0,  1,  2,  3,  4,  5,  6,  7,  8, -1,  9, 10, 11, 12, -1,
    13, 14, 15, 16, 17, -1, 18, 19, 20, 21, 22, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
};

//here the substitutions for 20 amino acids
static  int amino_acids_trans_20[256] = {
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1,  0, -1,  1,  2,  3,  4,  5,  6,  7, -1,  8,  9, 10, 11, -1,
    12, 13, 14, 15, 16, -1, 17, 18, -1, 19, -1, -1, -1, -1, -1, -1,
    -1,  0, -1,  1,  2,  3,  4,  5,  6,  7, -1,  8,  9, 10, 11, -1,
    12, 13, 14, 15, 16, -1, 17, 18, -1, 19, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
};

// set size of acids amino acid residue is 20
static  char amino_acids_20[20] = {
    'A', 'C', 'D', 'E', 'F', 'G', 'H',
    'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R',
    'S', 'T', 'V', 'W', 'Y'
};



#endif
